@@ -1,3 +1,3 @@
 SHLIB      = libg++.sl
-BUILD_LIBS = $(SHLIB)
+BUILD_LIBS = $(ARLIB) $(SHLIB)
 SHFLAGS    = $(LIBCFLAGS)
