@@ -14,11 +14,9 @@ class Entry_Handler
 {
   
 private:
-#if ( __GNUG__ > 1) /* Initial number of file entries per class. */
+  /* Initial number of file entries per class. */
   static const int    default_entries;  
-#else
-  static const int    default_entries = 50;
-#endif
+
                int    max_entry_length; /* Size of largest filename. */
                int    total_entries;    /* Total number of filenames. */
                int    entries;          /* Current number of filenames. */
@@ -49,6 +47,12 @@ Entry_Handler::entry_number (void)
   return entries;
 }
 
+inline void *
+operator new (size_t size, void *orig, int number)
+{
+  return realloc (orig, size * number);
+}
+
 inline void 
 Entry_Handler::add_entry (char *entry_name, int length)
 {
@@ -58,7 +62,12 @@ Entry_Handler::add_entry (char *entry_name, int length)
 #else
     buf = new (buf, total_entries *= 2) char *;
 #endif
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
   max_entry_length >?= length;
+#else
+  if (length > max_entry_length)
+    max_entry_length = length;
+#endif
   buf[entries++] = strcpy (new char[length + 1], entry_name);
 }
 
