@@ -16,8 +16,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU GPERF; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+along with GNU GPERF; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include <stdio.h>
 #include <std.h>
@@ -36,7 +36,7 @@ int Trace::nesting;
    strings are allocated. */
 
 void *
-operator new (long size)
+operator new (size_t size)
 {
   T (Trace t ("operator new");)
   static char *buf_start = 0;          /* Large array used to reduce calls to NEW. */
@@ -57,7 +57,9 @@ operator new (long size)
   if (buf_start + size >= buf_end)
     {
       buf_size *= 2;
-      if (buf_start = (char *)malloc (buf_size >?= size))
+      if (buf_size < size)
+	buf_size = size;
+      if (buf_start = (char *)malloc (buf_size))
         buf_end = buf_start + buf_size;
       else
         Std_Err::report_error ("Virtual memory failed at %s, %s in function %s\n%a", __FILE__, __LINE__, "operator new", 1);
