@@ -12,7 +12,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* 
@@ -32,6 +32,8 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <string.h>
 #include <strstream.h>
 
+#undef OK
+
 void BitSet::error(const char* msg) const
 {
   (*lib_error_handler)("BitSet", msg);
@@ -77,7 +79,7 @@ inline static BitSetRep* BSnew(int newlen)
   if (allocsiz >= MAXBitSetRep_SIZE * sizeof(short))
     (*lib_error_handler)("BitSet", "Requested length out of range");
     
-  BitSetRep* rep = (BitSetRep *) new char[allocsiz];
+  BitSetRep* rep = new (operator new (allocsiz)) BitSetRep;
   memset(rep, 0, allocsiz);
   rep->sz = (allocsiz - sizeof(BitSetRep) + sizeof(short)) / sizeof(short);
   return rep;
@@ -124,7 +126,13 @@ BitSetRep* BitSetresize(BitSetRep* old, int newlen)
     delete old;
   }
   else
-    rep = old;
+    {
+      rep = old;
+      if (rep->len < newlen)
+	memset(&rep->s[rep->len],
+	       rep->virt ? ONES : 0,
+	       (newlen - rep->len) * sizeof(short));
+    }
 
   rep->len = newlen;
 
@@ -687,7 +695,7 @@ int BitSet::next(int p, int b) const
   }
 }
 
-int BitSet::previous(int p, int b) const
+int BitSet::prev(int p, int b) const
 {
   if (--p < 0)
     return -1;
@@ -770,7 +778,7 @@ int BitSet::last(int b) const
   if (b == rep->virt)
     return -1;
   else
-    return previous((rep->len) * BITSETBITS, b);
+    return prev((rep->len) * BITSETBITS, b);
 }
 
 
@@ -788,7 +796,7 @@ const char* BitSettoa(const BitSet& x, char f, char t, char star)
   return fmtbase;
 }
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 BitSet shorttoBitSet(unsigned short w) return r
 {
@@ -938,11 +946,11 @@ ostream& operator << (ostream& s, const BitSet& x)
   return s;
 }
 
-void BitSet::printon(ostream& s, char f, char t, char star) const
+void BitSet::printon(ostream& os, char f, char t, char star) const
 // FIXME:  Does not respect s.width()!
 {
   trim(rep);
-  register streambuf* sb = s.rdbuf();
+  register streambuf* sb = os.rdbuf();
   const unsigned short* s = rep->s;
   const unsigned short* top = &(s[rep->len - 1]);
 
