@@ -13,7 +13,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #ifndef _BitSet_h
@@ -26,6 +26,8 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <iostream.h>
 #include <limits.h>
 
+#undef OK
+
 #define BITSETBITS  (sizeof(short) * CHAR_BIT)
 
 struct BitSetRep
@@ -59,8 +61,9 @@ protected:
                      BitSetBit(BitSet* v, int p);
                      BitSetBit(const BitSetBit& b);
                     ~BitSetBit();
-                     operator int();
+                     operator int() const;
   int                operator = (int b);
+  int                operator = (const BitSetBit& b);
 };
 
 class BitSet
@@ -77,7 +80,7 @@ public:
 
                     ~BitSet();
 
-  void               operator =  (const BitSet& y);
+  BitSet&            operator =  (const BitSet& y);
 
 // equality & subset tests
 
@@ -91,10 +94,10 @@ public:
 
 // operations on self
 
-  void               operator |= (const BitSet& y);
-  void               operator &= (const BitSet& y);
-  void               operator -= (const BitSet& y);
-  void               operator ^= (const BitSet& y);
+  BitSet&            operator |= (const BitSet& y);
+  BitSet&            operator &= (const BitSet& y);
+  BitSet&            operator -= (const BitSet& y);
+  BitSet&            operator ^= (const BitSet& y);
 
   void               complement();
 
@@ -164,14 +167,6 @@ public:
 
 typedef BitSet BitSetTmp;
 
-
-  BitSet      operator |  (const BitSet& x, const BitSet& y);
-  BitSet      operator &  (const BitSet& x, const BitSet& y);
-  BitSet      operator -  (const BitSet& x, const BitSet& y);
-  BitSet      operator ^  (const BitSet& x, const BitSet& y);
-
-  BitSet      operator ~  (const BitSet& x);
-
 // These are inlined regardless of optimization
 
 inline int BitSet_index(int l)
@@ -191,9 +186,10 @@ inline BitSet::BitSet(const BitSet& x) :rep(BitSetcopy(0, x.rep)) {}
 
 inline BitSet::~BitSet() { if (rep != &_nilBitSetRep) delete rep; }
 
-inline void BitSet::operator =  (const BitSet& y)
+inline BitSet& BitSet::operator =  (const BitSet& y)
 { 
   rep = BitSetcopy(rep, y.rep);
+  return *this;
 }
 
 inline int operator != (const BitSet& x, const BitSet& y) { return !(x == y); }
@@ -227,7 +223,7 @@ inline void complement(const BitSet& x, BitSet& r)
   r.rep = BitSetcmpl(x.rep, r.rep);
 }
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 inline BitSet operator & (const BitSet& x, const BitSet& y) return r
 {
@@ -283,24 +279,28 @@ inline BitSet operator ~ (const BitSet& x)
 
 #endif
 
-inline void BitSet::operator &= (const BitSet& y)
+inline BitSet& BitSet::operator &= (const BitSet& y)
 {
   and(*this, y, *this);
+  return *this;
 }
 
-inline void BitSet::operator |= (const BitSet& y)
+inline BitSet& BitSet::operator |= (const BitSet& y)
 {
   or(*this, y, *this);
+  return *this;
 }
 
-inline void BitSet::operator ^= (const BitSet& y)
+inline BitSet& BitSet::operator ^= (const BitSet& y)
 {
   xor(*this, y, *this);
+  return *this;
 }
 
-inline void BitSet::operator -= (const BitSet& y)
+inline BitSet& BitSet::operator -= (const BitSet& y)
 {
   diff(*this, y, *this);
+  return *this;
 }
 
 
@@ -342,7 +342,7 @@ inline BitSetBit::BitSetBit(BitSet* v, int p)
 
 inline BitSetBit::~BitSetBit() {}
 
-inline BitSetBit::operator int()
+inline BitSetBit::operator int() const
 {
   return src->test(pos);
 }
@@ -352,6 +352,13 @@ inline int BitSetBit::operator = (int b)
   if (b) src->set(pos); else src->clear(pos); return b;
 }
 
+inline int BitSetBit::operator = (const BitSetBit& b)
+{
+  int i = (int)b;
+  *this = i;
+  return i;
+}
+
 inline BitSetBit BitSet::operator [] (int i)
 {
   if (i < 0) error("illegal bit index");
