@@ -13,7 +13,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #ifndef _BitString_h
@@ -26,16 +26,19 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <stream.h>
 #include <limits.h>
 
-#define BITSTRBITS  (sizeof(short) * CHAR_BIT)
+#undef OK
+
+#include <bitprims.h>
+#define BITSTRBITS  _BS_BITS_PER_WORD
 
 struct BitStrRep
 {
   unsigned int    len;          // length in bits
   unsigned short  sz;           // allocated slots
-  unsigned short  s[1];         // bits start here
+  _BS_word        s[1];         // bits start here
 };
 
-extern BitStrRep*  BStr_alloc(BitStrRep*, const unsigned short*, int, int,int);
+extern BitStrRep*  BStr_alloc(BitStrRep*, const _BS_word*, int, int,int);
 extern BitStrRep*  BStr_resize(BitStrRep*, int);
 extern BitStrRep*  BStr_copy(BitStrRep*, const BitStrRep*);
 extern BitStrRep*  cmpl(const BitStrRep*, BitStrRep*);
@@ -63,8 +66,6 @@ protected:
                    ~BitStrBit();
                     operator unsigned int() const;
   int               operator =  (unsigned int b);
-  int               operator == (unsigned int b) const ;
-  int               operator != (unsigned int b) const ;
 };
 
 class BitSubString
@@ -83,8 +84,8 @@ protected:
 public:
                     ~BitSubString();
 
-  void              operator =  (const BitString&);
-  void              operator =  (const BitSubString&);
+  BitSubString&     operator =  (const BitString&);
+  BitSubString&     operator =  (const BitSubString&);
 
   int               length() const;
   int               empty() const;
@@ -99,8 +100,8 @@ class BitString
 protected:
   BitStrRep*         rep;
 
-  int                search(int, int, const unsigned short*, int, int) const;
-  int                match(int, int, int, const unsigned short*,int,int) const;
+  int                search(int, int, const _BS_word*, int, int) const;
+  int                match(int, int, int, const _BS_word*,int,int) const;
   BitSubString       _substr(int first, int l);
 
 public:
@@ -112,9 +113,9 @@ public:
 
                     ~BitString();
 
-  void               operator =  (unsigned int bit);
-  void               operator =  (const BitString& y);
-  void               operator =  (const BitSubString& y);
+  BitString&         operator =  (unsigned int bit);
+  BitString&         operator =  (const BitString& y);
+  BitString&         operator =  (const BitSubString& y);
 
 // equality & subset tests
 
@@ -144,14 +145,14 @@ public:
 // assignment-based operators
 // (constuctive versions decalred inline below
 
-  void               operator |= (const BitString&);
-  void               operator &= (const BitString&);
-  void               operator -= (const BitString&);
-  void               operator ^= (const BitString&);
-  void               operator += (const BitString&);
-  void               operator += (unsigned int b);
-  void               operator <<=(int s);
-  void               operator >>=(int s);
+  BitString&         operator |= (const BitString&);
+  BitString&         operator &= (const BitString&);
+  BitString&         operator -= (const BitString&);
+  BitString&         operator ^= (const BitString&);
+  BitString&         operator += (const BitString&);
+  BitString&         operator += (unsigned int b);
+  BitString&         operator <<=(int s);
+  BitString&         operator >>=(int s);
 
   void               complement();
 
@@ -183,7 +184,9 @@ public:
   int                last(unsigned int b = 1) const;
 
   int                next(int pos, unsigned int b = 1) const;
-  int                previous(int pos, unsigned int b = 1) const;
+  int                prev(int pos, unsigned int b = 1) const;
+  int                previous(int pos, unsigned int b = 1) const
+    { return prev(pos, b); } /* Obsolete synonym */
 
 // searching & matching
 
@@ -277,13 +280,13 @@ public:
                     ~BitPattern();
 
   friend const char* BitPatterntoa(const BitPattern& p, 
-                                  char f='0',char t='1',char x='X');
+                                 char f/*='0'*/,char t/*='1'*/,char x/*='X'*/);
   void		     printon(ostream&, char f='0',char t='1',char x='X') const;
   friend BitPattern atoBitPattern(const char* s, char f,char t, char x);
   friend ostream&   operator << (ostream& s, const BitPattern&);
 
-  int               search(const unsigned short*, int, int) const;
-  int               match(const unsigned short* xs, int, int, int) const;
+  int               search(const _BS_word*, int, int) const;
+  int               match(const _BS_word* xs, int, int, int) const;
 
   int               OK() const;
 };
@@ -327,54 +330,49 @@ inline BitString::~BitString()
   if (rep != &_nilBitStrRep) delete rep;
 }
 
-#if defined(__GNUG__) && !defined(NO_NRV)
-
-inline BitString shorttoBitString(unsigned short w) return r
-{ 
-  r.rep = BStr_alloc(0, &w, 0, BITSTRBITS, BITSTRBITS);
-}
-
-inline BitString longtoBitString(unsigned long w) return r
-{ 
-  unsigned short u[2];
-  u[0] = w & ((unsigned short)(~(0)));
-  u[1] = w >> BITSTRBITS;
-  r.rep = BStr_alloc(0, &u[0], 0, 2*BITSTRBITS, 2*BITSTRBITS);
-}
-
-#else
-
 inline BitString shorttoBitString(unsigned short w) 
 { 
-  BitString r; r.rep = BStr_alloc(0, &w, 0, BITSTRBITS, BITSTRBITS); return r;
+  BitString r;
+  _BS_word ww = w;
+#if _BS_BIGENDIAN
+  abort();
+#endif
+  r.rep = BStr_alloc(0, &ww, 0, 8 * sizeof(short), 8 * sizeof(short));
+  return r;
 }
 
 inline BitString longtoBitString(unsigned long w) 
 { 
   BitString r;
+#if 1
+  _BS_word u = w;
+  r.rep = BStr_alloc(0, &u, 0, BITSTRBITS, BITSTRBITS);
+#else
   unsigned short u[2];
   u[0] = w & ((unsigned short)(~(0)));
   u[1] = w >> BITSTRBITS;
   r.rep = BStr_alloc(0, &u[0], 0, 2*BITSTRBITS, 2*BITSTRBITS);
+#endif
   return r;
 }
 
-#endif
-
-inline void BitString::operator =  (const BitString& y)
+inline BitString& BitString::operator =  (const BitString& y)
 { 
   rep = BStr_copy(rep, y.rep);
+  return *this;
 }
 
-inline void BitString::operator = (unsigned int b)
+inline BitString& BitString::operator = (unsigned int b)
 { 
-  unsigned short bit = b;
+  _BS_word bit = b;
   rep = BStr_alloc(rep, &bit, 0, 1, 1);
+  return *this;
 }
 
-inline void BitString::operator=(const BitSubString&  y)
+inline BitString&  BitString::operator=(const BitSubString&  y)
 {
   rep = BStr_alloc(rep, y.S.rep->s, y.pos, y.pos+y.len, y.len);
+  return *this;
 }
 
 inline BitSubString::BitSubString(const BitSubString& x) 
@@ -445,45 +443,53 @@ inline void complement(const BitString& x, BitString& r)
 // operators
 
 
-inline void BitString::operator &= (const BitString& y)
+inline BitString& BitString::operator &= (const BitString& y)
 {
   and(*this, y, *this);
+  return *this;
 }
 
 
-inline void BitString::operator |= (const BitString& y)
+inline BitString& BitString::operator |= (const BitString& y)
 {
   or(*this, y, *this);
+  return *this;
 }
 
-inline void BitString::operator ^= (const BitString& y)
+inline BitString& BitString::operator ^= (const BitString& y)
 {
   xor(*this, y, *this);
+  return *this;
 }
 
-inline void BitString::operator <<= (int y)
+inline BitString& BitString::operator <<= (int y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
-inline void BitString::operator >>= (int y)
+inline BitString& BitString::operator >>= (int y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-inline void BitString::operator -= (const BitString& y)
+inline BitString& BitString::operator -= (const BitString& y)
 {
   diff(*this, y, *this);
+  return *this;
 }
 
-inline void BitString::operator += (const BitString& y)
+inline BitString& BitString::operator += (const BitString& y)
 {
   cat(*this, y, *this);
+  return *this;
 }
 
-inline void BitString::operator += (unsigned int y)
+inline BitString& BitString::operator += (unsigned int y)
 {
   cat(*this, y, *this);
+  return *this;
 }
 
 inline void BitString::complement()
@@ -491,7 +497,7 @@ inline void BitString::complement()
   ::complement(*this, *this);
 }
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 inline BitString  operator & (const BitString& x, const BitString& y) return r
 {
@@ -691,7 +697,7 @@ inline int BitString::first(unsigned int b) const
 
 inline int BitString::last(unsigned int b) const
 {
-  return previous(rep->len, b);
+  return prev(rep->len, b);
 }
 
 inline int BitString::index(unsigned int bit, int startpos) const
@@ -699,13 +705,13 @@ inline int BitString::index(unsigned int bit, int startpos) const
   if (startpos >= 0)
     return next(startpos - 1, bit);
   else
-    return previous(rep->len + startpos + 1, bit);
+    return prev(rep->len + startpos + 1, bit);
 }
 
 inline void BitString::right_trim(unsigned int b) 
 {
   int nb = (b == 0)? 1 : 0;
-  rep = BStr_resize(rep, previous(rep->len, nb) + 1);
+  rep = BStr_resize(rep, prev(rep->len, nb) + 1);
 }
 
 inline void BitString::left_trim(unsigned int b)
@@ -740,16 +746,6 @@ inline int BitStrBit::operator = (unsigned int b)
   src.assign(pos, b); return b;
 }
 
-inline int BitStrBit::operator == (unsigned int b) const
-{
-  return src.test(pos) == b;
-}
-
-inline int BitStrBit::operator != (unsigned int b) const
-{
-  return src.test(pos) != b;
-}
-
 inline BitStrBit BitString::operator [] (int i)
 {
   if ((unsigned)(i) >= rep->len) error("illegal bit index");
