@@ -12,7 +12,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #ifdef __GNUG__
@@ -220,7 +220,7 @@ istream& operator >> (istream& s, Complex& x)
 #else
   if (!s.ipfx(0))
   {
-    s.set(ios::failbit); // Redundant if using GNU iostreams.
+    s.clear(ios::failbit|s.rdstate()); // Redundant if using GNU iostreams.
     return s;
   }
 #endif
@@ -242,7 +242,7 @@ istream& operator >> (istream& s, Complex& x)
     else
       i = 0;
     if (ch != ')')
-      s.clear(_fail);
+      s.clear(ios::failbit);
   }
   else
   {
