@@ -14,7 +14,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #ifndef _Integer_h
@@ -23,7 +23,9 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #endif
 #define _Integer_h 1
 
-#include <stream.h>
+#include <iostream.h>
+
+#undef OK
 
 struct IntRep                    // internal Integer representations
 {
@@ -71,9 +73,7 @@ extern char*    cvtItoa(const IntRep* x, char* fmt, int& fmtlen, int base,
                         char fillchar, char Xcase, int showpos);
 extern IntRep*  atoIntRep(const char* s, int base = 10);
 extern long     Itolong(const IntRep*);
-extern double   Itodouble(const IntRep*);
 extern int      Iislong(const IntRep*);
-extern int      Iisdouble(const IntRep*);
 extern long     lg(const IntRep*);
 
 extern IntRep _ZeroRep, _OneRep, _MinusOneRep;
@@ -84,51 +84,51 @@ protected:
   IntRep*         rep;
 public:
                   Integer();
+                  Integer(int);
                   Integer(long);
                   Integer(unsigned long);
                   Integer(IntRep*);
                   Integer(const Integer&);
 
                   ~Integer();
-
-  void            operator =  (const Integer&);
-  void            operator =  (long);
+  Integer&        operator =  (const Integer&);
+  Integer&        operator =  (long);
 
 // unary operations to self
 
-  void            operator ++ ();
-  void            operator -- ();
+  Integer&        operator ++ ();
+  Integer&        operator -- ();
   void            negate();          // negate in-place
   void            abs();             // absolute-value in-place
   void            complement();      // bitwise complement in-place
 
 // assignment-based operations
 
-  void            operator += (const Integer&);
-  void            operator -= (const Integer&);
-  void            operator *= (const Integer&);
-  void            operator /= (const Integer&);
-  void            operator %= (const Integer&);
-  void            operator <<=(const Integer&);
-  void            operator >>=(const Integer&);
-  void            operator &= (const Integer&);
-  void            operator |= (const Integer&);
-  void            operator ^= (const Integer&);
+  Integer&        operator += (const Integer&);
+  Integer&        operator -= (const Integer&);
+  Integer&        operator *= (const Integer&);
+  Integer&        operator /= (const Integer&);
+  Integer&        operator %= (const Integer&);
+  Integer&        operator <<=(const Integer&);
+  Integer&        operator >>=(const Integer&);
+  Integer&        operator &= (const Integer&);
+  Integer&        operator |= (const Integer&);
+  Integer&        operator ^= (const Integer&);
 
-  void            operator += (long);
-  void            operator -= (long);
-  void            operator *= (long);
-  void            operator /= (long);
-  void            operator %= (long);
-  void            operator <<=(long);
-  void            operator >>=(long);
-  void            operator &= (long);
-  void            operator |= (long);
-  void            operator ^= (long);
+  Integer&        operator += (long);
+  Integer&        operator -= (long);
+  Integer&        operator *= (long);
+  Integer&        operator /= (long);
+  Integer&        operator %= (long);
+  Integer&        operator <<=(long);
+  Integer&        operator >>=(long);
+  Integer&        operator &= (long);
+  Integer&        operator |= (long);
+  Integer&        operator ^= (long);
 
 // (constructive binary operations are inlined below)
 
-#ifdef __GNUG__
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
   friend Integer operator <? (const Integer& x, const Integer& y); // min
   friend Integer operator >? (const Integer& x, const Integer& y); // max
 #endif
@@ -196,11 +196,11 @@ public:
 
 // coercion & conversion
 
-  int             fits_in_long() const;
+  int             fits_in_long() const { return Iislong(rep); }
   int             fits_in_double() const;
 
-                  operator long() const;
-                  operator double() const;
+  long		  as_long() const { return Itolong(rep); }
+  double	  as_double() const;
 
   friend char*    Itoa(const Integer& x, int base = 10, int width = 0);
   friend Integer  atoI(const char* s, int base = 10);
@@ -281,6 +281,8 @@ inline Integer::Integer() :rep(&_ZeroRep) {}
 
 inline Integer::Integer(IntRep* r) :rep(r) {}
 
+inline Integer::Integer(int y) :rep(Icopy_long(0, (long)y)) {}
+
 inline Integer::Integer(long y) :rep(Icopy_long(0, y)) {}
 
 inline Integer::Integer(unsigned long y) :rep(Icopy_ulong(0, y)) {}
@@ -289,19 +291,16 @@ inline Integer::Integer(const Integer&  y) :rep(Icopy(0, y.rep)) {}
 
 inline Integer::~Integer() { if (rep && !STATIC_IntRep(rep)) delete rep; }
 
-inline void  Integer::operator = (const Integer&  y)
+inline Integer&  Integer::operator = (const Integer&  y)
 {
   rep = Icopy(rep, y.rep);
+  return *this;
 }
 
-inline void Integer::operator = (long y)
+inline Integer& Integer::operator = (long y)
 {
-  rep = Icopy_long(rep, y); 
-}
-
-inline Integer::operator long() const
-{ 
-  return Itolong(rep);
+  rep = Icopy_long(rep, y);
+  return *this;
 }
 
 inline int Integer::initialized() const
@@ -309,21 +308,6 @@ inline int Integer::initialized() const
   return rep != 0;
 }
 
-inline int Integer::fits_in_long() const
-{
-  return Iislong(rep);
-}
-
-inline Integer::operator double() const
-{ 
-  return Itodouble(rep);
-}
-
-inline int Integer::fits_in_double() const
-{
-  return Iisdouble(rep);
-}
-
 // procedural versions
 
 inline int compare(const Integer& x, const Integer& y)
@@ -408,7 +392,7 @@ inline void  rshift(const Integer& x, const Integer& y, Integer& dest)
 
 inline void  pow(const Integer& x, const Integer& y, Integer& dest)
 {
-  dest.rep = power(x.rep, long(y), dest.rep); // not incorrect
+  dest.rep = power(x.rep, Itolong(y.rep), dest.rep); // not incorrect
 }
 
 inline void  add(const Integer& x, long y, Integer& dest)
@@ -575,127 +559,136 @@ inline int operator >= (const Integer&  x, long y)
 }
 
 
-inline void  Integer::operator += (const Integer& y)
+inline Integer&  Integer::operator += (const Integer& y)
 {
   add(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator += (long y)
+inline Integer&  Integer::operator += (long y)
 {
   add(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator ++ ()
+inline Integer& Integer::operator ++ ()
 {
   add(*this, 1, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator -= (const Integer& y)
+inline Integer& Integer::operator -= (const Integer& y)
 {
   sub(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator -= (long y)
+inline Integer& Integer::operator -= (long y)
 {
   sub(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator -- ()
+inline Integer& Integer::operator -- ()
 {
   add(*this, -1, *this);
+  return *this;
 }
 
 
 
-inline void Integer::operator *= (const Integer& y)
+inline Integer& Integer::operator *= (const Integer& y)
 {
   mul(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator *= (long y)
+inline Integer& Integer::operator *= (long y)
 {
   mul(*this, y, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator &= (const Integer& y)
+inline Integer& Integer::operator &= (const Integer& y)
 {
   and(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator &= (long y)
+inline Integer& Integer::operator &= (long y)
 {
   and(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator |= (const Integer& y)
+inline Integer& Integer::operator |= (const Integer& y)
 {
   or(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator |= (long y)
+inline Integer& Integer::operator |= (long y)
 {
   or(*this, y, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator ^= (const Integer& y)
+inline Integer& Integer::operator ^= (const Integer& y)
 {
   xor(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator ^= (long y)
+inline Integer& Integer::operator ^= (long y)
 {
   xor(*this, y, *this);
+  return *this;
 }
 
 
 
-inline void Integer::operator /= (const Integer& y)
+inline Integer& Integer::operator /= (const Integer& y)
 {
   div(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator /= (long y)
+inline Integer& Integer::operator /= (long y)
 {
   div(*this, y, *this);
+  return *this;
 }
 
 
-inline void Integer::operator %= (const Integer& y)
-{
-  *this = *this % y; // mod(*this, y, *this) doesn't work.
-}
-
-inline void Integer::operator %= (long y)
-{
-  *this = *this % y; // mod(*this, y, *this) doesn't work.
-}
-
-
-inline void Integer::operator <<= (const Integer&  y)
+inline Integer& Integer::operator <<= (const Integer&  y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator <<= (long  y)
+inline Integer& Integer::operator <<= (long  y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
 
-inline void Integer::operator >>= (const Integer&  y)
+inline Integer& Integer::operator >>= (const Integer&  y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator >>= (long y)
+inline Integer& Integer::operator >>= (long y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-#ifdef __GNUG__
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
 inline Integer operator <? (const Integer& x, const Integer& y)
 {
   return (compare(x.rep, y.rep) <= 0) ? x : y;
@@ -754,7 +747,7 @@ inline long lg(const Integer& x)
 
 // constructive operations 
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 inline Integer  operator +  (const Integer& x, const Integer& y) return r
 {
@@ -1113,4 +1106,16 @@ inline Integer  gcd(const Integer& x, const Integer& y)
 }
 
 #endif  /* NO_NRV */
+
+inline Integer& Integer::operator %= (const Integer& y)
+{
+  *this = *this % y; // mod(*this, y, *this) doesn't work.
+  return *this;
+}
+
+inline Integer& Integer::operator %= (long y)
+{
+  *this = *this % y; // mod(*this, y, *this) doesn't work.
+  return *this;
+}
 #endif /* !_Integer_h */
