@@ -12,7 +12,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* 
@@ -29,9 +29,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <new.h>
 #include <builtin.h>
 
-// extern "C" {
-#include <regex.h>
-// }
+#undef OK
 
 void String::error(const char* msg) const
 {
@@ -133,7 +131,7 @@ inline static StrRep* Snew(int newsiz)
   if (allocsiz >= MAXStrRep_SIZE)
     (*lib_error_handler)("String", "Requested length out of range");
     
-  StrRep* rep = (StrRep *) new char[allocsiz];
+  StrRep* rep = new (operator new (allocsiz)) StrRep;
   rep->sz = allocsiz - sizeof(StrRep);
   return rep;
 }
@@ -171,7 +169,8 @@ StrRep* Salloc(StrRep* old, const char* src, int srclen, int newlen)
 // generally be faster in the long run to get new space & copy
 // than to call realloc
 
-StrRep* Sresize(StrRep* old, int newlen)
+static StrRep*
+Sresize(StrRep* old, int newlen)
 {
   if (old == &_nilStrRep) old = 0;
   StrRep* rep;
@@ -191,6 +190,14 @@ StrRep* Sresize(StrRep* old, int newlen)
   return rep;
 }
 
+void
+String::alloc (int newsize)
+{
+  unsigned short old_len = rep->len;
+  rep = Sresize(rep, newsize);
+  rep->len = old_len;
+}
+
 // like allocate, but we know that src is a StrRep
 
 StrRep* Scopy(StrRep* old, const StrRep* s)
@@ -960,56 +967,39 @@ int split(const String& src, String results[], int n, const Regex& r)
 
 
 #if defined(__GNUG__) && !defined(_G_NO_NRV)
-
-String join(String src[], int n, const String& separator) return x;
-{
-  String sep = separator;
-  int xlen = 0;
-  for (int i = 0; i < n; ++i)
-    xlen += src[i].length();
-  xlen += (n - 1) * sep.length();
-
-  x.alloc(xlen);
-
-  int j = 0;
-  
-  for (i = 0; i < n - 1; ++i)
-  {
-    ncopy(src[i].chars(), &(x.rep->s[j]), src[i].length());
-    j += src[i].length();
-    ncopy(sep.chars(), &(x.rep->s[j]), sep.length());
-    j += sep.length();
-  }
-  ncopy0(src[i].chars(), &(x.rep->s[j]), src[i].length());
-}
-
-#else
-
-String join(String src[], int n, const String& separator)
-{
-  String x;
-  String sep = separator;
-  int xlen = 0;
-  for (int i = 0; i < n; ++i)
-    xlen += src[i].length();
-  xlen += (n - 1) * sep.length();
-
-  x.alloc(xlen);
-
-  int j = 0;
-  
-  for (i = 0; i < n - 1; ++i)
-  {
-    ncopy(src[i].chars(), &(x.rep->s[j]), src[i].length());
-    j += src[i].length();
-    ncopy(sep.chars(), &(x.rep->s[j]), sep.length());
-    j += sep.length();
-  }
-  ncopy0(src[i].chars(), &(x.rep->s[j]), src[i].length());
-  return x;
-}
-
+#define RETURN(r) return
+#define RETURNS(r) return r;
+#define RETURN_OBJECT(TYPE, NAME) /* nothing */
+#else /* _G_NO_NRV */
+#define RETURN(r) return r
+#define RETURNS(r) /* nothing */
+#define RETURN_OBJECT(TYPE, NAME) TYPE NAME;
 #endif
+
+String join(String src[], int n, const String& separator) RETURNS(x)
+{
+  RETURN_OBJECT(String,x)
+  String sep = separator;
+  int xlen = 0;
+  int i;
+  for (i = 0; i < n; ++i)
+    xlen += src[i].length();
+  xlen += (n - 1) * sep.length();
+
+  x.rep = Sresize (x.rep, xlen);
+
+  int j = 0;
+  
+  for (i = 0; i < n - 1; ++i)
+  {
+    ncopy(src[i].chars(), &(x.rep->s[j]), src[i].length());
+    j += src[i].length();
+    ncopy(sep.chars(), &(x.rep->s[j]), sep.length());
+    j += sep.length();
+  }
+  ncopy0(src[i].chars(), &(x.rep->s[j]), src[i].length());
+  RETURN(x);
+}
   
 /*
  misc
@@ -1119,7 +1109,8 @@ String common_prefix(const String& x, const String& y, int startpos) return r;
   const char* topx = &(xchars[x.length()]);
   const char* ys = &(ychars[startpos]);
   const char* topy = &(ychars[y.length()]);
-  for (int l = 0; xs < topx && ys < topy && *xs++ == *ys++; ++l);
+  int l;
+  for (l = 0; xs < topx && ys < topy && *xs++ == *ys++; ++l);
   r.rep = Salloc(r.rep, ss, l, l);
 }
 
@@ -1131,7 +1122,8 @@ String common_suffix(const String& x, const String& y, int startpos) return r;
   const char* botx = xchars;
   const char* ys = &(ychars[y.length() + startpos]);
   const char* boty = ychars;
-  for (int l = 0; xs >= botx && ys >= boty && *xs == *ys ; --xs, --ys, ++l);
+  int l;
+  for (l = 0; xs >= botx && ys >= boty && *xs == *ys ; --xs, --ys, ++l);
   r.rep = Salloc(r.rep, ++xs, l, l);
 }
 
