@@ -64,19 +64,20 @@ check: tests
 .PHONY: check-tGetOpt
 EOF
 
-for TEST in ${TESTS0} ${TESTS1} ; do
+for TEST in ${TESTS0} ${TESTS1} tiLList ; do
   echo ".PHONY: check-${TEST}"
-  echo "check-${TEST}: ${TEST}" '$(srcdir)'"/${TEST}.inp"
-  echo "	./${TEST} < "'$(srcdir)'"/${TEST}.inp > ${TEST}.out 2>&1"
+  if [ -f ${srcdir}/${TEST}.inp ] ; then
+    echo "check-${TEST}: ${TEST}" '$(srcdir)'"/${TEST}.inp"
+    echo "	./${TEST} < "'$(srcdir)'"/${TEST}.inp > ${TEST}.out 2>&1"
+  else
+    echo "check-${TEST}: ${TEST}"
+    echo "	./${TEST} > ${TEST}.out 2>&1"
+  fi
   echo '	diff -b $(srcdir)/'"${TEST}.exp ${TEST}.out"
 done
 
 cat <<'EOF'
 
-check-tiLList: tiLList $(srcdir)/tLList.inp
-	./tiLList < $(srcdir)/tLList.inp > tiLList.out 2>&1
-	diff -b $(srcdir)/tLList.exp tiLList.out
-
 check-tGetOpt: tGetOpt $(srcdir)/tGetOpt.inp
 	./tGetOpt -abc -de10 -2000 -h3i \
 	  <$(srcdir)/tGetOpt.inp >tGetOpt.out 2>&1
@@ -99,7 +100,7 @@ tests checktests: clean_tests test_h tCurses \
   check-tStack check-tQueue check-tDeque check-tPQ \
   check-tSet check-tBag check-tMap $(CHECK_TEMPLATES)
 	./test_h
-	echo "(Must run tCurses manually from tty)"
+	@echo "(Must run tCurses manually from tty)"
 
 check-templates: check-tiLList
 
