@@ -234,7 +234,7 @@ int main()
   cout << "\n";
 
   cout << "clear bits in g (reverse order):\n";
-  for (p = g.last(0); p >= 0; p = g.prev(p, 0)) 
+  for (int p = g.last(0); p >= 0; p = g.prev(p, 0)) 
   {
     assert(g[p] == 0);
     cout << p << " ";
