@@ -54,7 +54,7 @@ void XPtest ()
   assert(d.rear() == (SIZE-2));
   assert(!d.full());
   intXPDeque d1(SIZE/2);
-  for (i = (SIZE-1); i >= 0; --i)
+  for (int i = (SIZE-1); i >= 0; --i)
   {
     int x;
     if (i % 2 == 0)
@@ -109,7 +109,7 @@ void DLtest ()
   assert(d.rear() == (SIZE-2));
   assert(!d.full());
   intDLDeque d1;
-  for (i = (SIZE-1); i >= 0; --i)
+  for (int i = (SIZE-1); i >= 0; --i)
   {
     int x;
     if (i % 2 == 0)
