@@ -31,7 +31,8 @@ void printset(intSet& a)
   int maxprint = 20;
   cout << "[";
   int k = 0;
-  for (Pix i = a.first(); i != 0 && k < maxprint; a.next(i),++k) 
+  Pix i;
+  for (i = a.first(); i != 0 && k < maxprint; a.next(i),++k) 
     cout << a(i) << " ";
   if (i != 0) cout << "...]\n";
   else cout << "]\n";
@@ -525,7 +526,7 @@ void BSTtest()
   for (int j = 1; j <= SIZE; ++j) assert(a.contains(j));
   a.balance();
   assert(a.OK());
-  for ( j = 1; j <= SIZE; ++j) assert(a.contains(j));
+  for (int j = 1; j <= SIZE; ++j) assert(a.contains(j));
   intBSTSet b;
   add(odds, b);
   assert(b.length() == SIZE);
@@ -693,8 +694,7 @@ void Splaytest()
 }
 
 
-
-main(int argc, char** argv)
+int main(int argc, char** argv)
 {
   if (argc > 1)
   {
@@ -739,4 +739,6 @@ main(int argc, char** argv)
   start_timer();
   cout << "Splaytest\n"; Splaytest();
   if (ptimes) cout << "\ntime = " << return_elapsed_time(0.0) << "\n";
+
+  return 0;
 }
