@@ -1,5 +1,5 @@
 /* Demangler for GNU C++ 
-   Copyright 1989, 1991 Free Software Foundation, Inc.
+   Copyright 1989, 1991, 1994, 1995 Free Software Foundation, Inc.
    Written by James Clark (jjc@jclark.uucp)
    Rewritten by Fred Fish (fnf@cygnus.com) for ARM and Lucid demangling
    
@@ -25,17 +25,17 @@ Cambridge, MA 02139, USA.  */
    realloc except that they generate a fatal error if there is no
    available memory. */
 
-#include <demangle.h>
-#undef CURRENT_DEMANGLING_STYLE
-#define CURRENT_DEMANGLING_STYLE work->options
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
 
-extern char *xmalloc PARAMS((long));
-extern char *xrealloc PARAMS((PTR, long));
-extern char *strstr PARAMS ((const char *, const char *));
-extern void free PARAMS((PTR));
+#include <demangle.h>
+#undef CURRENT_DEMANGLING_STYLE
+#define CURRENT_DEMANGLING_STYLE work->options
+
+extern char *xmalloc PARAMS((unsigned));
+extern char *xrealloc PARAMS((char *, unsigned));
+extern char *strstr ();
 
 /* In order to allow a single demangler executable to demangle strings
    using various common values of CPLUS_MARKER, as well as any specific
@@ -86,16 +86,18 @@ struct work_stuff
 #define PRINT_ANSI_QUALIFIERS (work -> options & DMGL_ANSI)
 #define PRINT_ARG_TYPES       (work -> options & DMGL_PARAMS)
 
-static CONST struct optable
+static const struct optable
 {
-  CONST char *in;
-  CONST char *out;
+  const char *in;
+  const char *out;
   int flags;
 } optable[] = {
   {"nw",	  " new",	DMGL_ANSI},	/* new (1.92,	 ansi) */
   {"dl",	  " delete",	DMGL_ANSI},	/* new (1.92,	 ansi) */
   {"new",	  " new",	0},		/* old (1.91,	 and 1.x) */
   {"delete",	  " delete",	0},		/* old (1.91,	 and 1.x) */
+  {"vn",	  " new []",	DMGL_ANSI},	/* GNU, pending ansi */
+  {"vd",	  " delete []",	DMGL_ANSI},	/* GNU, pending ansi */
   {"as",	  "=",		DMGL_ANSI},	/* ansi */
   {"ne",	  "!=",		DMGL_ANSI},	/* old, ansi */
   {"eq",	  "==",		DMGL_ANSI},	/* old,	ansi */
@@ -194,34 +196,34 @@ mop_up PARAMS ((struct work_stuff *, string *, int));
 
 #if 0
 static int
-demangle_method_args PARAMS ((struct work_stuff *work, CONST char **, string *));
+demangle_method_args PARAMS ((struct work_stuff *work, const char **, string *));
 #endif
 
 static int
-demangle_template PARAMS ((struct work_stuff *work, CONST char **, string *,
+demangle_template PARAMS ((struct work_stuff *work, const char **, string *,
 			   string *));
 
 static int
-demangle_qualified PARAMS ((struct work_stuff *, CONST char **, string *,
+demangle_qualified PARAMS ((struct work_stuff *, const char **, string *,
 			    int, int));
 
 static int
-demangle_class PARAMS ((struct work_stuff *, CONST char **, string *));
+demangle_class PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-demangle_fund_type PARAMS ((struct work_stuff *, CONST char **, string *));
+demangle_fund_type PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-demangle_signature PARAMS ((struct work_stuff *, CONST char **, string *));
+demangle_signature PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-demangle_prefix PARAMS ((struct work_stuff *, CONST char **, string *));
+demangle_prefix PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-gnu_special PARAMS ((struct work_stuff *, CONST char **, string *));
+gnu_special PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-arm_special PARAMS ((struct work_stuff *, CONST char **, string *));
+arm_special PARAMS ((struct work_stuff *, const char **, string *));
 
 static void
 string_need PARAMS ((string *, int));
@@ -241,41 +243,41 @@ string_empty PARAMS ((string *));
 #endif
 
 static void
-string_append PARAMS ((string *, CONST char *));
+string_append PARAMS ((string *, const char *));
 
 static void
 string_appends PARAMS ((string *, string *));
 
 static void
-string_appendn PARAMS ((string *, CONST char *, int));
+string_appendn PARAMS ((string *, const char *, int));
 
 static void
-string_prepend PARAMS ((string *, CONST char *));
+string_prepend PARAMS ((string *, const char *));
 
 static void
-string_prependn PARAMS ((string *, CONST char *, int));
+string_prependn PARAMS ((string *, const char *, int));
 
 static int
-get_count PARAMS ((CONST char **, int *));
+get_count PARAMS ((const char **, int *));
 
 static int
-consume_count PARAMS ((CONST char **));
+consume_count PARAMS ((const char **));
 
 static int
-demangle_args PARAMS ((struct work_stuff *, CONST char **, string *));
+demangle_args PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-do_type PARAMS ((struct work_stuff *, CONST char **, string *));
+do_type PARAMS ((struct work_stuff *, const char **, string *));
 
 static int
-do_arg PARAMS ((struct work_stuff *, CONST char **, string *));
+do_arg PARAMS ((struct work_stuff *, const char **, string *));
 
 static void
-demangle_function_name PARAMS ((struct work_stuff *, CONST char **, string *,
-				CONST char *));
+demangle_function_name PARAMS ((struct work_stuff *, const char **, string *,
+				const char *));
 
 static void
-remember_type PARAMS ((struct work_stuff *, CONST char *, int));
+remember_type PARAMS ((struct work_stuff *, const char *, int));
 
 static void
 forget_types PARAMS ((struct work_stuff *));
@@ -290,7 +292,7 @@ string_prepends PARAMS ((string *, string *));
 
 static int
 consume_count (type)
-    CONST char **type;
+    const char **type;
 {
     int count = 0;
 
@@ -312,7 +314,7 @@ cplus_demangle_opname (opname, result, options)
   int len, i, len1, ret;
   string type;
   struct work_stuff work[1];
-  CONST char *tem;
+  const char *tem;
 
   len = strlen(opname);
   result[0] = '\0';
@@ -455,7 +457,7 @@ cplus_mangle_opname (opname, options)
    characters. */
 
 int cplus_match (mangled, text, text_len)
-     CONST char *mangled;
+     const char *mangled;
      char *text;
      int text_len;
 {
@@ -496,7 +498,7 @@ int cplus_match (mangled, text, text_len)
 
 char *
 cplus_demangle (mangled, options)
-     CONST char *mangled;
+     const char *mangled;
      int options;
 {
   string decl;
@@ -612,13 +614,13 @@ DESCRIPTION
 static int
 demangle_signature (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int success = 1;
   int func_done = 0;
   int expect_func = 0;
-  CONST char *oldmangled = NULL;
+  const char *oldmangled = NULL;
   string trawname;
   string tname;
 
@@ -705,7 +707,15 @@ demangle_signature (work, mangled, declp)
 	    /* G++ Template */
 	    string_init(&trawname); 
 	    string_init(&tname);
+            if (oldmangled == NULL)
+              {
+                oldmangled = *mangled;
+              }
 	    success = demangle_template (work, mangled, &tname, &trawname);
+            if (success)
+              {
+                remember_type (work, oldmangled, *mangled - oldmangled);
+              }
 	    string_append(&tname, "::");
 	    string_prepends(declp, &tname);
   	    if (work -> destructor & 1)
@@ -721,6 +731,7 @@ demangle_signature (work, mangled, declp)
               }
 	    string_delete(&trawname);
 	    string_delete(&tname);
+	    oldmangled = NULL;
 	    expect_func = 1;
 	    break;
 
@@ -751,7 +762,9 @@ demangle_signature (work, mangled, declp)
 	      }
 	    break;
 	}
+/*
       if (AUTO_DEMANGLING || GNU_DEMANGLING)
+*/
 	{
 	  if (success && expect_func)
 	    {
@@ -789,7 +802,7 @@ demangle_signature (work, mangled, declp)
 static int
 demangle_method_args (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int success = 0;
@@ -812,7 +825,7 @@ demangle_method_args (work, mangled, declp)
 static int
 demangle_template (work, mangled, tname, trawname)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *tname;
      string *trawname;
 {
@@ -820,19 +833,21 @@ demangle_template (work, mangled, tname, trawname)
   int is_pointer;
   int is_real;
   int is_integral;
+  int is_char;
+  int is_bool;
   int r;
   int need_comma = 0;
   int success = 0;
   int done;
-  CONST char *old_p;
-  CONST char *start;
+  const char *old_p;
+  const char *start;
   int symbol_len;
   string temp;
 
   (*mangled)++;
   start = *mangled;
   /* get template name */
-  if ((r = consume_count (mangled)) == 0)
+  if ((r = consume_count (mangled)) == 0 || strlen (*mangled) < r)
     {
       return (0);
     }
@@ -875,24 +890,30 @@ demangle_template (work, mangled, tname, trawname)
 	  is_pointer = 0;
 	  is_real = 0;
 	  is_integral = 0;
+          is_char = 0;
 	  done = 0;
 	  /* temp is initialized in do_type */
 	  success = do_type (work, mangled, &temp);
+/*
 	  if (success)
 	    {
 	      string_appends (tname, &temp);
 	    }
+*/
 	  string_delete(&temp);
 	  if (!success)
 	    {
 	      break;
 	    }
+/*
 	  string_append (tname, "=");
+*/
 	  while (*old_p && !done)
 	    {	
 	      switch (*old_p)
 		{
 		  case 'P':
+		  case 'p':
 		  case 'R':
 		    done = is_pointer = 1;
 		    break;
@@ -905,7 +926,9 @@ demangle_template (work, mangled, tname, trawname)
 		  case 'O':	/* ??? */
 		    old_p++;
 		    continue;
-		  case 'Q':	/* repetition of following */
+		  case 'Q':	/* qualified name */
+                    done = is_integral = 1;
+                    break;
 		  case 'T':	/* remembered type */
 		    abort ();
 		    break;
@@ -916,10 +939,15 @@ demangle_template (work, mangled, tname, trawname)
 		  case 'l':	/* long */
 		  case 'i':	/* int */
 		  case 's':	/* short */
-		  case 'c':	/* char */
 		  case 'w':	/* wchar_t */
 		    done = is_integral = 1;
 		    break;
+		  case 'b':	/* bool */
+		    done = is_bool = 1;
+		    break;
+		  case 'c':	/* char */
+		    done = is_char = 1;
+		    break;
 		  case 'r':	/* long double */
 		  case 'd':	/* double */
 		  case 'f':	/* float */
@@ -945,6 +973,37 @@ demangle_template (work, mangled, tname, trawname)
 		  (*mangled)++;
 		}
 	    }
+	  else if (is_char)
+	    {
+            char tmp[2];
+            int val;
+              if (**mangled == 'm')
+                {
+                  string_appendn (tname, "-", 1);
+                  (*mangled)++;
+                }
+	      string_appendn (tname, "'", 1);
+              val = consume_count(mangled);
+	      if (val == 0)
+		{
+		  success = 0;
+		  break;
+                }
+              tmp[0] = (char)val;
+              tmp[1] = '\0';
+              string_appendn (tname, &tmp[0], 1);
+	      string_appendn (tname, "'", 1);
+	    }
+	  else if (is_bool)
+	    {
+	      int val = consume_count (mangled);
+	      if (val == 0)
+		string_appendn (tname, "false", 5);
+	      else if (val == 1)
+		string_appendn (tname, "true", 4);
+	      else
+		success = 0;
+	    }
 	  else if (is_real)
 	    {
 	      if (**mangled == 'm')
@@ -991,6 +1050,8 @@ demangle_template (work, mangled, tname, trawname)
 	}
       need_comma = 1;
     }
+  if (tname->p[-1] == '>')
+    string_append (tname, " ");
   string_append (tname, ">");
   
 /*
@@ -1012,9 +1073,9 @@ demangle_template (work, mangled, tname, trawname)
 static int
 arm_pt (work, mangled, n, anchor, args)
      struct work_stuff *work;
-     CONST char *mangled;
+     const char *mangled;
      int n;
-     CONST char **anchor, **args;
+     const char **anchor, **args;
 {
   /* ARM template? */
   if (ARM_DEMANGLING && (*anchor = strstr(mangled, "__pt__")))
@@ -1031,10 +1092,46 @@ arm_pt (work, mangled, n, anchor, args)
   return 0;
 }
 
+static void
+demangle_arm_pt (work, mangled, n, declp)
+     struct work_stuff *work;
+     const char **mangled;
+     int n;
+     string *declp;
+{
+  const char *p;
+  const char *args;
+  const char *e = *mangled + n;
+
+  /* ARM template? */
+  if (arm_pt (work, *mangled, n, &p, &args))
+  {
+    string arg;
+    string_init (&arg);
+    string_appendn (declp, *mangled, p - *mangled);
+    string_append (declp, "<");
+    /* should do error checking here */
+    while (args < e) {
+      string_clear (&arg);
+      do_type (work, &args, &arg);
+      string_appends (declp, &arg);
+      string_append (declp, ",");
+    }
+    string_delete (&arg);
+    --declp->p;
+    string_append (declp, ">");
+  }
+  else
+  {
+    string_appendn (declp, *mangled, n);
+  }
+  *mangled += n;
+}
+
 static int
 demangle_class_name (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int n;
@@ -1042,35 +1139,11 @@ demangle_class_name (work, mangled, declp)
 
   n = consume_count (mangled);
   if (strlen (*mangled) >= n)
-    {
-      CONST char *p;
-      CONST char *args;
-      CONST char *e = *mangled + n;
-      /* ARM template? */
-      if (arm_pt (work, *mangled, n, &p, &args))
-	{
-	  string arg;
-	  string_init (&arg);
-	  string_appendn (declp, *mangled, p - *mangled);
-	  string_append (declp, "<");
-	  /* should do error checking here */
-	  while (args < e) {
-	    string_clear (&arg);
-	    do_type (work, &args, &arg);
-	    string_appends (declp, &arg);
-	    string_append (declp, ",");
-	  }
-	  string_delete (&arg);
-	  --declp->p;
-	  string_append (declp, ">");
-        }
-      else
-	{
-	  string_appendn (declp, *mangled, n);
-        }
-      *mangled += n;
-      success = 1;
-    }
+  {
+    demangle_arm_pt (work, mangled, n, declp);
+    success = 1;
+  }
+
   return (success);
 }
 
@@ -1095,7 +1168,7 @@ DESCRIPTION
 	On exit, it points to the next token after the mangled class on
 	success, or the first unconsumed token on failure.
 
-	If the CONSTRUCTOR or DESTRUCTOR flags are set in WORK, then
+	If the constRUCTOR or DESTRUCTOR flags are set in WORK, then
 	we are demangling a constructor or destructor.  In this case
 	we prepend "class::class" or "class::~class" to DECLP.
 
@@ -1112,7 +1185,7 @@ DESCRIPTION
 static int
 demangle_class (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int success = 0;
@@ -1176,11 +1249,11 @@ DESCRIPTION
 static int
 demangle_prefix (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int success = 1;
-  CONST char *scan;
+  const char *scan;
   int i;
 
   if (strlen(*mangled) >= 11 && strncmp(*mangled, "_GLOBAL_", 8) == 0)
@@ -1193,12 +1266,16 @@ demangle_prefix (work, mangled, declp)
 	      /* it's a GNU global destructor to be executed at program exit */
 	      (*mangled) += 11;
 	      work->destructor = 2;
+	      if (gnu_special (work, mangled, declp))
+		return success;
 	    }
 	  else if ((*mangled)[9] == 'I')
 	    {
 	      /* it's a GNU global constructor to be executed at program init */
 	      (*mangled) += 11;
 	      work->constructor = 2;
+	      if (gnu_special (work, mangled, declp))
+		return success;
 	    }
 	}
     }
@@ -1267,9 +1344,12 @@ demangle_prefix (work, mangled, declp)
 	}
       else
 	{
-      /* A GNU style constructor starts with "__[0-9Qt]. */
-          work -> constructor += 1;
-          *mangled = scan + 2;
+	  /* A GNU style constructor starts with __[0-9Qt].  But cfront uses
+	     names like __Q2_3foo3bar for nested type names.  So don't accept
+	     this style of constructor for cfront demangling.  */
+	  if (!(LUCID_DEMANGLING || ARM_DEMANGLING))
+	    work -> constructor += 1;
+	  *mangled = scan + 2;
 	}
     }
   else if ((scan == *mangled) && !isdigit (scan[2]) && (scan[2] != 't'))
@@ -1296,6 +1376,14 @@ demangle_prefix (work, mangled, declp)
 	    }
 	}
     }
+  else if (ARM_DEMANGLING && scan[2] == 'p' && scan[3] == 't')
+    {
+      /* Cfront-style parameterized type.  Handled later as a signature. */
+      success = 1;
+
+      /* ARM template? */
+      demangle_arm_pt (work, mangled, strlen (*mangled), declp);
+    }
   else if (*(scan + 2) != '\0')
     {
       /* Mangled name does not start with "__" but does have one somewhere
@@ -1339,20 +1427,22 @@ DESCRIPTION
 		_$_3foo		(destructor for class foo)
 		_vt$foo		(foo virtual table)
 		_vt$foo$bar	(foo::bar virtual table)
+		__vt_foo	(foo virtual table, new style with thunks)
 		_3foo$varname	(static data member)
 		_Q22rs2tu$vw	(static data member)
 		__t6vector1Zii	(constructor with template)
+		__thunk_4__$_7ostream (virtual function thunk)
  */
 
 static int
 gnu_special (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int n;
   int success = 1;
-  CONST char *p;
+  const char *p;
 
   if ((*mangled)[0] == '_'
       && strchr (cplus_markers, (*mangled)[1]) != NULL
@@ -1363,35 +1453,62 @@ gnu_special (work, mangled, declp)
       work -> destructor += 1;
     }
   else if ((*mangled)[0] == '_'
-	   && (*mangled)[1] == 'v'
-	   && (*mangled)[2] == 't'
-	   && strchr (cplus_markers, (*mangled)[3]) != NULL)
+	   && (((*mangled)[1] == '_'
+		&& (*mangled)[2] == 'v'
+		&& (*mangled)[3] == 't'
+		&& (*mangled)[4] == '_')
+	     || ((*mangled)[1] == 'v'
+		 && (*mangled)[2] == 't'
+		 && strchr (cplus_markers, (*mangled)[3]) != NULL)))
     {
       /* Found a GNU style virtual table, get past "_vt<CPLUS_MARKER>"
          and create the decl.  Note that we consume the entire mangled
 	 input string, which means that demangle_signature has no work
 	 to do. */
-      (*mangled) += 4;
+      if ((*mangled)[2] == 'v')
+	(*mangled) += 5; /* New style, with thunks: "__vt_" */
+      else
+	(*mangled) += 4; /* Old style, no thunks: "_vt<CPLUS_MARKER>" */
       while (**mangled != '\0')
 	{
-	  if (isdigit(*mangled[0]))
+	  p = strpbrk (*mangled, cplus_markers);
+	  switch (**mangled)
 	    {
-	      n = consume_count(mangled);
+	    case 'Q':
+	      success = demangle_qualified (work, mangled, declp, 0, 1);
+	      break;
+	    case 't':
+	      success = demangle_template (work, mangled, declp, 0);
+	      break;
+	    default:
+	      if (isdigit(*mangled[0]))
+		{
+		  n = consume_count(mangled);
+		}
+	      else
+		{
+		  n = strcspn (*mangled, cplus_markers);
+		}
+	      string_appendn (declp, *mangled, n);
+	      (*mangled) += n;
+	    }
+
+	  if (success && ((p == NULL) || (p == *mangled)))
+	    {
+	      if (p != NULL)
+		{
+		  string_append (declp, "::");
+		  (*mangled)++;
+		}
 	    }
 	  else
 	    {
-	      n = strcspn (*mangled, cplus_markers);
-	    }
-	  string_appendn (declp, *mangled, n);
-	  (*mangled) += n;
-	  
-	  if (**mangled != '\0')
-	    {
-	      string_append (declp, "::");
-	      (*mangled)++;
+	      success = 0;
+	      break;
 	    }
 	}
-      string_append (declp, " virtual table");
+      if (success)
+	string_append (declp, " virtual table");
     }
   else if ((*mangled)[0] == '_'
 	   && (strchr("0123456789Qt", (*mangled)[1]) != NULL)
@@ -1427,6 +1544,25 @@ gnu_special (work, mangled, declp)
 	  success = 0;
 	}
     }
+  else if (strncmp (*mangled, "__thunk_", 8) == 0)
+    {
+      int delta = ((*mangled) += 8, consume_count (mangled));
+      char *method = cplus_demangle (++*mangled, work->options);
+      if (method)
+	{
+	  char buf[50];
+	  sprintf (buf, "virtual function thunk (delta:%d) for ", -delta);
+	  string_append (declp, buf);
+	  string_append (declp, method);
+	  free (method);
+	  n = strlen (*mangled);
+	  (*mangled) += n;
+	}
+      else
+	{
+	  success = 0;
+	}
+    }
   else
     {
       success = 0;
@@ -1460,12 +1596,12 @@ DESCRIPTION
 static int
 arm_special (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   int n;
   int success = 1;
-  CONST char *scan;
+  const char *scan;
 
   if (strncmp (*mangled, ARM_VTABLE_STRING, ARM_VTABLE_STRLEN) == 0)
     {
@@ -1543,7 +1679,7 @@ BUGS
 static int
 demangle_qualified (work, mangled, result, isfuncname, append)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *result;
      int isfuncname;
      int append;
@@ -1551,7 +1687,7 @@ demangle_qualified (work, mangled, result, isfuncname, append)
   int qualifiers;
   int namelength;
   int success = 1;
-  CONST char *p;
+  const char *p;
   char num[2];
   string temp;
 
@@ -1614,6 +1750,8 @@ demangle_qualified (work, mangled, result, isfuncname, append)
 
   while (qualifiers-- > 0)
     {
+      if (*mangled[0] == '_') 
+	*mangled = *mangled + 1;
       if (*mangled[0] == 't')
 	{
 	  success = demangle_template(work, mangled, &temp, 0);
@@ -1690,10 +1828,10 @@ DESCRIPTION
 
 static int
 get_count (type, count)
-     CONST char **type;
+     const char **type;
      int *count;
 {
-  CONST char *p;
+  const char *p;
   int n;
 
   if (!isdigit (**type))
@@ -1730,14 +1868,14 @@ get_count (type, count)
 static int
 do_type (work, mangled, result)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *result;
 {
   int n;
   int done;
   int success;
   string decl;
-  CONST char *remembered_type;
+  const char *remembered_type;
   int constp;
   int volatilep;
 
@@ -1754,6 +1892,7 @@ do_type (work, mangled, result)
 
 	/* A pointer type */
 	case 'P':
+	case 'p':
 	  (*mangled)++;
 	  string_prepend (&decl, "*");
 	  break;
@@ -1767,7 +1906,7 @@ do_type (work, mangled, result)
 	/* An array */
 	case 'A':
 	  {
-	    CONST char *p = ++(*mangled);
+	    const char *p = ++(*mangled);
 
 	    string_prepend (&decl, "(");
 	    string_append (&decl, ")[");
@@ -1961,7 +2100,7 @@ do_type (work, mangled, result)
 static int
 demangle_fund_type (work, mangled, result)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *result;
 {
   int done = 0;
@@ -2037,6 +2176,11 @@ demangle_fund_type (work, mangled, result)
 	APPEND_BLANK (result);
 	string_append (result, "short");
 	break;
+      case 'b':
+	(*mangled)++;
+	APPEND_BLANK (result);
+	string_append (result, "bool");
+	break;
       case 'c':
 	(*mangled)++;
 	APPEND_BLANK (result);
@@ -2103,10 +2247,10 @@ demangle_fund_type (work, mangled, result)
 static int
 do_arg (work, mangled, result)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *result;
 {
-  CONST char *start = *mangled;
+  const char *start = *mangled;
 
   if (!do_type (work, mangled, result))
     {
@@ -2122,7 +2266,7 @@ do_arg (work, mangled, result)
 static void
 remember_type (work, start, len)
      struct work_stuff *work;
-     CONST char *start;
+     const char *start;
      int len;
 {
   char *tem;
@@ -2213,14 +2357,14 @@ forget_types (work)
 static int
 demangle_args (work, mangled, declp)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
 {
   string arg;
   int need_comma = 0;
   int r;
   int t;
-  CONST char *tem;
+  const char *tem;
   char temptype;
 
   if (PRINT_ARG_TYPES)
@@ -2249,9 +2393,26 @@ demangle_args (work, mangled, declp)
 	    {
 	      r = 1;
 	    }
-	  if (!get_count (mangled, &t))
+          if (ARM_DEMANGLING && work -> ntypes >= 10)
+            {
+              /* If we have 10 or more types we might have more than a 1 digit
+                 index so we'll have to consume the whole count here. This
+                 will loose if the next thing is a type name preceeded by a
+                 count but it's impossible to demangle that case properly
+                 anyway. Eg if we already have 12 types is T12Pc "(..., type1,
+                 anyway. Eg if we already have 12 types is T12Pc "(..., type1,
+                 Pc, ...)"  or "(..., type12, char *, ...)" */
+              if ((t = consume_count(mangled)) == 0)
+                {
+                  return (0);
+                }
+            }
+          else
 	    {
-	      return (0);
+	      if (!get_count (mangled, &t))
+	    	{
+	          return (0);
+	    	}
 	    }
 	  if (LUCID_DEMANGLING || ARM_DEMANGLING)
 	    {
@@ -2324,14 +2485,14 @@ demangle_args (work, mangled, declp)
 static void
 demangle_function_name (work, mangled, declp, scan)
      struct work_stuff *work;
-     CONST char **mangled;
+     const char **mangled;
      string *declp;
-     CONST char *scan;
+     const char *scan;
 {
   int i;
   int len;
   string type;
-  CONST char *tem;
+  const char *tem;
 
   string_appendn (declp, (*mangled), scan - (*mangled));
   string_need (declp, 1);
@@ -2539,7 +2700,7 @@ string_empty (s)
 static void
 string_append (p, s)
      string *p;
-     CONST char *s;
+     const char *s;
 {
   int n;
   if (s == NULL || *s == '\0')
@@ -2568,7 +2729,7 @@ string_appends (p, s)
 static void
 string_appendn (p, s, n)
      string *p;
-     CONST char *s;
+     const char *s;
      int n;
 {
   if (n != 0)
@@ -2582,7 +2743,7 @@ string_appendn (p, s, n)
 static void
 string_prepend (p, s)
      string *p;
-     CONST char *s;
+     const char *s;
 {
   if (s != NULL && *s != '\0')
     {
@@ -2603,7 +2764,7 @@ string_prepends (p, s)
 static void
 string_prependn (p, s, n)
      string *p;
-     CONST char *s;
+     const char *s;
      int n;
 {
   char *q;
@@ -2620,9 +2781,10 @@ string_prependn (p, s, n)
     }
 }
 
-/* To generate a standalone demangler program for testing purposes, just
-   compile and link this file with -DMAIN.   When run, it demangles each
-   command line arg, or each stdin string, and prints the result on stdout. */
+/* To generate a standalone demangler program for testing purposes,
+   just compile and link this file with -DMAIN and libiberty.a.  When
+   run, it demangles each command line arg, or each stdin string, and
+   prints the result on stdout. */
 
 #ifdef MAIN
 
@@ -2644,39 +2806,10 @@ demangle_it (mangled_name)
     }
 }
 
-char *
-xmalloc (size)
-    long size;
-{
-  char * newmem;
-
-  if ((newmem = (char *) malloc ((int) size)) == NULL)
-    {
-      fprintf (stderr, "\nCan't allocate %u bytes\n", size);
-      exit (1);
-    }
-  return (newmem);
-}
-
-char *
-xrealloc (oldmem, size)
-    PTR oldmem;
-    long size;
-{
-  char * newmem;
-
-  if ((newmem = (char *) realloc ((char *) oldmem, (int) size)) == NULL)
-    {
-      fprintf (stderr, "\nCan't reallocate %u bytes\n", size);
-      exit (1);
-    }
-  return (newmem);
-}
-
 #include "getopt.h"
 
 static char *program_name;
-extern char *program_version;
+static char *program_version = VERSION;
 
 static void
 usage (stream, status)
@@ -2684,8 +2817,9 @@ usage (stream, status)
      int status;
 {    
   fprintf (stream, "\
-Usage: %s [-_] [-s {gnu,lucid,arm}] [--strip-underscores]\n\
-       [--format={gnu,lucid,arm}] [--help] [--version] [arg...]\n",
+Usage: %s [-_] [-n] [-s {gnu,lucid,arm}] [--strip-underscores]\n\
+       [--no-strip-underscores] [--format={gnu,lucid,arm}]\n\
+       [--help] [--version] [arg...]\n",
 	   program_name);
   exit (status);
 }
@@ -2693,7 +2827,7 @@ Usage: %s [-_] [-s {gnu,lucid,arm}] [--strip-underscores]\n\
 #define MBUF_SIZE 512
 char mbuffer[MBUF_SIZE];
 
-/* Defined in the automatically-generated ../binutils/underscore.c. */
+/* Defined in the automatically-generated underscore.c. */
 extern int prepends_underscore;
 
 int strip_underscore = 0;
@@ -2702,6 +2836,7 @@ static struct option long_options[] = {
   {"strip-underscores", no_argument, 0, '_'},
   {"format", required_argument, 0, 's'},
   {"help", no_argument, 0, 'h'},
+  {"no-strip-underscores", no_argument, 0, 'n'},
   {"version", no_argument, 0, 'v'},
   {0, no_argument, 0, 0}
 };
@@ -2715,9 +2850,10 @@ main (argc, argv)
   int c;
 
   program_name = argv[0];
+
   strip_underscore = prepends_underscore;
 
-  while ((c = getopt_long (argc, argv, "_s:", long_options, (int *) 0)) != EOF)
+  while ((c = getopt_long (argc, argv, "_ns:", long_options, (int *) 0)) != EOF)
     {
       switch (c)
 	{
@@ -2726,6 +2862,9 @@ main (argc, argv)
 	    break;
 	  case 'h':
 	    usage (stdout, 0);
+	  case 'n':
+	    strip_underscore = 0;
+	    break;
 	  case 'v':
 	    printf ("GNU %s version %s\n", program_name, program_version);
 	    exit (0);
@@ -2778,18 +2917,31 @@ main (argc, argv)
 	    }
 	  if (i > 0)
 	    {
-	      int skip_first = strip_underscore && i > 1 && mbuffer[0] == '_';
+	      int skip_first = 0;
+
+	      if (mbuffer[0] == '.')
+		++skip_first;
+	      if (strip_underscore && mbuffer[skip_first] == '_')
+		++skip_first;
+
+	      if (skip_first > i)
+		skip_first = i;
+
 	      mbuffer[i] = 0;
 	      
-	      result = cplus_demangle (mbuffer+skip_first,
+	      result = cplus_demangle (mbuffer + skip_first,
 				       DMGL_PARAMS | DMGL_ANSI);
 	      if (result)
 		{
+		  if (mbuffer[0] == '.')
+		    putc ('.', stdout);
 		  fputs (result, stdout);
 		  free (result);
 		}
 	      else
-		fputs (mbuffer + skip_first, stdout);
+		fputs (mbuffer, stdout);
+
+	      fflush (stdout);
 	    }
 	  if (c == EOF)
 	    break;
@@ -2800,4 +2952,35 @@ main (argc, argv)
   exit (0);
 }
 
+static void
+fatal (str)
+     char *str;
+{
+  fprintf (stderr, "%s: %s\n", program_name, str);
+  exit (1);
+}
+
+char * malloc ();
+char * realloc ();
+
+char *
+xmalloc (size)
+     unsigned size;
+{
+  register char *value = (char *) malloc (size);
+  if (value == 0)
+    fatal ("virtual memory exhausted");
+  return value;
+}
+
+char *
+xrealloc (ptr, size)
+     char *ptr;
+     unsigned size;
+{
+  register char *value = (char *) realloc (ptr, size);
+  if (value == 0)
+    fatal ("virtual memory exhausted");
+  return value;
+}
 #endif	/* main */
