@@ -1,12 +1,15 @@
 #include <ansidecl.h>
+#include "alloca-conf.h"
 
 #ifdef __STDC__
 #include <stddef.h>
+#define clock_t unsigned long
 #define DEF(NAME, RETURN_TYPE, ARGLIST, ARGS) extern RETURN_TYPE NAME (ARGS);
 #define DEFFUNC(NAME, RETURN_TYPE, ARGLIST, ARGS) extern RETURN_TYPE NAME (ARGS);
 #else
 #define void int
 #define size_t unsigned long
+#define clock_t unsigned long
 #define DEF(NAME, RETURN_TYPE, ARGLIST, ARGS) extern RETURN_TYPE NAME ();
 #define DEFFUNC(NAME, RETURN_TYPE, ARGLIST, ARGS) extern RETURN_TYPE NAME ();
 #endif
