@@ -1,5 +1,5 @@
 /* This is part of libio/iostream, providing -*- C++ -*- input/output.
-Copyright (C) 1993 Free Software Foundation
+Copyright (C) 1993, 1995 Free Software Foundation
 
 This file is part of the GNU IO Library.  This library is free
 software; you can redistribute it and/or modify it under the
@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -41,16 +41,18 @@ filebuf::filebuf()
   _IO_file_init(this);
 }
 
+#if !_IO_UNIFIED_JUMPTABLES
 /* This is like "new filebuf()", but it uses the _IO_file_jump jumptable,
    for eficiency. */
 
 filebuf* filebuf::__new()
 {
   filebuf *fb = new filebuf;
-  fb->_jumps = &_IO_file_jumps;
+  _IO_JUMPS(fb) = &_IO_file_jumps;
   fb->_vtable() = builtinbuf_vtable;
   return fb;
 }
+#endif
 
 filebuf::filebuf(int fd)
 {
@@ -70,7 +72,7 @@ filebuf::~filebuf()
   if (_IO_file_is_open(this))
     _IO_do_flush (this);
   if (!(xflags() & _IO_DELETE_DONT_CLOSE))
-    _jumps->__close(this);
+    _IO_SYSCLOSE (this);
 }
 
 filebuf* filebuf::open(const char *filename, ios::openmode mode, int prot)
@@ -124,7 +126,7 @@ filebuf* filebuf::attach(int fd)
 
 streambuf* filebuf::setbuf(char* p, int len)
 {
-  return _IO_file_setbuf(this, p, len) == 0 ? this : NULL;
+  return (streambuf*)_IO_file_setbuf (this, p, len);
 }
 
 int filebuf::doallocate() { return _IO_file_doallocate(this); }
@@ -146,12 +148,12 @@ int filebuf::sync()
 
 streampos filebuf::seekoff(streamoff offset, _seek_dir dir, int mode)
 {
-  return _IO_file_seekoff (this, offset, convert_to_seekflags(dir, mode));
+  return _IO_file_seekoff (this, offset, dir, mode);
 }
 
 filebuf* filebuf::close()
 {
-  return (_IO_file_close_it(this) ? NULL : this);
+  return (_IO_file_close_it(this) ? (filebuf*)NULL : this);
 }
 
 streamsize filebuf::sys_read(char* buf, streamsize size)
