@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -64,8 +64,8 @@ void (*_IO_cleanup_registration_needed)();
  */
 
 int
-_IO_file_doallocate(fp)
-     register _IO_FILE *fp;
+DEFUN(_IO_file_doallocate, (fp),
+      register _IO_FILE *fp)
 {
   register _IO_size_t size;
   int couldbetty;
@@ -75,7 +75,7 @@ _IO_file_doallocate(fp)
   if (_IO_cleanup_registration_needed)
     (*_IO_cleanup_registration_needed)();
   
-  if (fp->_fileno < 0 || fp->_jumps->__stat(fp, &st) < 0)
+  if (fp->_fileno < 0 || _IO_SYSSTAT (fp, &st) < 0)
     {
       couldbetty = 0;
       size = _IO_BUFSIZ;
