@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -68,11 +68,11 @@ extern template class smanip<ios::fmtflags>;
 
 template<class TP>
 inline istream& operator>>(istream& i, const smanip<TP>& m)
-	{ (*m._f)(i, m._a); return i; }
+{ (*m._f)(i, m._a); return i; }
 
 template<class TP>
 inline ostream& operator<<(ostream& o, const smanip<TP>& m)
-	{ (*m._f)(o, m._a); return o;}
+{ (*m._f)(o, m._a); return o;}
 
 #ifdef __GNUG__
 extern template istream& operator>>(istream&, const smanip<int>&);
@@ -102,11 +102,13 @@ template <class TP> class imanip {
 public:
     imanip(istream& (*f)(istream&, TP), TP a) : _f(f), _a(a) {}
     //
-    friend 
-      istream& operator>>(istream& i, const imanip<TP>& m)
-	{ return (*m._f)( i, m._a); }
+    friend
+      istream& operator>>(istream& i, const imanip<TP>& m);
 };
 
+template <class TP>
+inline istream& operator>>(istream& i, const imanip<TP>& m)
+{ return (*m._f)( i, m._a); }
 
 //-----------------------------------------------------------------------------
 //	Output-Stream Manipulators
@@ -130,10 +132,12 @@ public:
     omanip(ostream& (*f)(ostream&, TP), TP a) : _f(f), _a(a) {}
     //
     friend
-      ostream& operator<<(ostream& o, omanip<TP>& m)
-	{ return (*m._f)(o, m._a); }
+      ostream& operator<<(ostream& o, const omanip<TP>& m);
 };
 
+template <class TP>
+inline ostream& operator<<(ostream& o, const omanip<TP>& m)
+{ return (*m._f)(o, m._a); }
 
 //-----------------------------------------------------------------------------
 //	Available Manipulators
