@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -74,10 +74,8 @@ typedef struct _IO_proc_file _IO_proc_file;
 static struct _IO_proc_file *proc_file_chain = NULL;
 
 _IO_FILE *
-_IO_proc_open(fp, command, mode)
-     _IO_FILE* fp;
-     const char *command;
-     const char *mode;
+DEFUN(_IO_proc_open, (fp, command, mode),
+      _IO_FILE* fp AND const char *command AND const char *mode)
 {
 #if _IO_HAVE_SYS_WAIT
   int read_or_write;
@@ -142,18 +140,19 @@ _IO_proc_open(fp, command, mode)
 }
 
 _IO_FILE *
-_IO_popen(command, mode)
-     const char *command;
-     const char *mode;
+DEFUN(_IO_popen, (command, mode),
+      const char *command AND const char *mode)
 {
   _IO_proc_file *fpx = (_IO_proc_file*)malloc(sizeof(_IO_proc_file));
   _IO_FILE *fp = (_IO_FILE*)fpx;
   if (fp == NULL)
     return NULL;
   _IO_init(fp, 0);
-  fp->_jumps = &_IO_proc_jumps;
+  _IO_JUMPS(fp) = &_IO_proc_jumps;
   _IO_file_init(fp);
+#if  !_IO_UNIFIED_JUMPTABLES
   ((struct _IO_FILE_plus*)fp)->vtable = NULL;
+#endif
   if (_IO_proc_open (fp, command, mode) != NULL)
     return fp;
   free (fpx);
@@ -161,8 +160,8 @@ _IO_popen(command, mode)
 }
 
 int
-_IO_proc_close(fp)
-     _IO_FILE *fp;
+DEFUN(_IO_proc_close, (fp),
+      _IO_FILE *fp)
 {
   /* This is not name-space clean. FIXME! */
 #if _IO_HAVE_SYS_WAIT
@@ -201,21 +200,22 @@ _IO_proc_close(fp)
 }
 
 struct _IO_jump_t _IO_proc_jumps = {
-  _IO_file_overflow,
-  _IO_file_underflow,
-  _IO_file_xsputn,
-  _IO_default_xsgetn,
-  _IO_file_read,
-  _IO_file_write,
-  _IO_file_doallocate,
-  _IO_default_pbackfail,
-  _IO_file_setbuf,
-  _IO_file_sync,
-  _IO_file_finish,
-  _IO_proc_close,
-  _IO_file_stat,
-  _IO_file_seek,
-  _IO_file_seekoff,
-  _IO_default_seekpos,
-  _IO_default_uflow
+  JUMP_INIT_DUMMY,
+  JUMP_INIT(finish, _IO_file_finish),
+  JUMP_INIT(overflow, _IO_file_overflow),
+  JUMP_INIT(underflow, _IO_file_underflow),
+  JUMP_INIT(uflow, _IO_default_uflow),
+  JUMP_INIT(pbackfail, _IO_default_pbackfail),
+  JUMP_INIT(xsputn, _IO_file_xsputn),
+  JUMP_INIT(xsgetn, _IO_default_xsgetn),
+  JUMP_INIT(seekoff, _IO_file_seekoff),
+  JUMP_INIT(seekpos, _IO_default_seekpos),
+  JUMP_INIT(setbuf, _IO_file_setbuf),
+  JUMP_INIT(sync, _IO_file_sync),
+  JUMP_INIT(doallocate, _IO_file_doallocate),
+  JUMP_INIT(read, _IO_file_read),
+  JUMP_INIT(write, _IO_file_write),
+  JUMP_INIT(seek, _IO_file_seek),
+  JUMP_INIT(close, _IO_proc_close),
+  JUMP_INIT(stat, _IO_file_stat)
 };
