@@ -1,11 +1,12 @@
 /* This should be replaced by whatever namespace-clean
    version of strerror you have available. */
 
-extern char *strerror();
+#include "libioP.h"
+extern char *strerror __P ((int));
 
 char *
-_IO_strerror(errnum)
-     int errnum;
+DEFUN(_IO_strerror, (errnum),
+      int errnum)
 {
   return strerror(errnum);
 }
