@@ -5,6 +5,6 @@ int
 fgetc(fp)
      FILE *fp;
 {
-  COERCE_FILE(fp);
+  CHECK_FILE(fp, EOF);
   return _IO_getc(fp);
 }
