@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -30,6 +30,8 @@ the executable file might be covered by the GNU General Public License. */
 
 /* #define _G_IO_THROW */ /* Not implemented:  ios::failure */
 
+#define _IO_NEW_STREAMS // new optimizated stream representation
+
 extern "C" {
 #include <libio.h>
 }
@@ -63,9 +65,6 @@ class ostream; class streambuf;
 #undef open
 #undef close
 
-extern "C" int __underflow(struct _IO_FILE*);
-extern "C" int __overflow(struct _IO_FILE*, int);
-
 typedef _IO_off_t streamoff;
 typedef _IO_fpos_t streampos;
 typedef _IO_ssize_t streamsize;
@@ -75,8 +74,6 @@ typedef unsigned char __iostate;
 
 struct _ios_fields
 { // The data members of an ios.
-  // Directly using _strbuf is dangerous, because the vtable
-  // pointer can be NULL.  Use rdbuf() when in doubt.
     streambuf *_strbuf;
     ostream* _tie;
     int _width;
@@ -117,6 +114,7 @@ enum open_mode {
 #endif
 
 class ios : public _ios_fields {
+  ios& operator=(ios&);  /* Not allowed! */
   public:
     typedef __fmtflags fmtflags;
     typedef int iostate;
@@ -145,8 +143,10 @@ class ios : public _ios_fields {
 	   showbase=_IO_SHOWBASE, showpoint=_IO_SHOWPOINT,
 	   uppercase=_IO_UPPERCASE, showpos=_IO_SHOWPOS,
 	   scientific=_IO_SCIENTIFIC, fixed=_IO_FIXED,
-	   unitbuf=_IO_UNITBUF, stdio=_IO_STDIO,
-	   dont_close=_IO_DONT_CLOSE // Don't delete streambuf on stream destruction
+	   unitbuf=_IO_UNITBUF, stdio=_IO_STDIO
+#ifndef _IO_NEW_STREAMS
+	   , dont_close=_IO_DONT_CLOSE // Don't delete streambuf on stream destruction
+#endif
 	   };
     enum { // Masks.
 	basefield=dec+oct+hex,
@@ -194,15 +194,6 @@ class ios : public _ios_fields {
     void _throw_failure() const { throw new ios::failure(this); }
 #else
     void _throw_failure() const { }
-#endif
-    streambuf* rdbuf() const { return _strbuf; }
-#ifdef _STREAM_COMPAT
-    void _IO_fix_vtable();  /* TEMPORARY - for binary compatibility */
-    void _IO_fix_vtable() const; /* TEMPORARY - for binary compatibility */
-#endif
-#if 0
-    streambuf* rdbuf(streambuf *_s) {
-      streambuf *_old = _strbuf; _strbuf = _s; return _old; }
 #endif
     void clear(iostate state = 0) {
 	_state = _strbuf ? state : state|badbit;
@@ -223,6 +214,10 @@ class ios : public _ios_fields {
 	_exceptions = enable;
 	if (_state & _exceptions) _throw_failure(); }
 
+    streambuf* rdbuf() const { return _strbuf; }
+    streambuf* rdbuf(streambuf *_s) {
+      streambuf *_old = _strbuf; _strbuf = _s; clear (); return _old; }
+
     static int sync_with_stdio(int on);
     static void sync_with_stdio() { sync_with_stdio(1); }
     static fmtflags bitalloc();
@@ -247,7 +242,7 @@ class ios : public _ios_fields {
     };
 
   protected:
-    ios(streambuf* sb = 0, ostream* tie_to = 0) { init(sb, tie_to); }
+    ios(streambuf* sb = 0, ostream* tie_to = 0);
     virtual ~ios();
     void init(streambuf* sb, ostream* tie = 0);
 };
@@ -277,8 +272,6 @@ class streammarker : private _IO_marker {
     int delta();
 };
 
-extern unsigned __adjust_column(unsigned start, const char *line, int count);
-
 struct streambuf : public _IO_FILE { // protected??
     friend class ios;
     friend class istream;
@@ -342,18 +335,26 @@ struct streambuf : public _IO_FILE { // protected??
   public:
     static int flush_all();
     static void flush_all_linebuffered(); // Flush all line buffered files.
-    virtual int underflow(); // Leave public for now
+    virtual ~streambuf();
     virtual int overflow(int c = EOF); // Leave public for now
-    virtual int doallocate();
-    streampos sseekoff(streamoff, _seek_dir, int mode=ios::in|ios::out);
-    streampos sseekpos(streampos pos, int mode = ios::in|ios::out);
-
+    virtual int underflow(); // Leave public for now
+    virtual int uflow(); // Leave public for now
+    virtual int pbackfail(int c);
+//    virtual int showmany ();
+    virtual streamsize xsputn(const char* s, streamsize n);
+    virtual streamsize xsgetn(char* s, streamsize n);
     virtual streampos seekoff(streamoff, _seek_dir, int mode=ios::in|ios::out);
     virtual streampos seekpos(streampos pos, int mode = ios::in|ios::out);
+
+    streampos sseekoff(streamoff, _seek_dir, int mode=ios::in|ios::out);
+    streampos sseekpos(streampos pos, int mode = ios::in|ios::out);
+    virtual streambuf* setbuf(char* p, int len);
+    virtual int sync();
+    virtual int doallocate();
+
     int seekmark(streammarker& mark, int delta = 0);
     int sputbackc(char c);
     int sungetc();
-    virtual ~streambuf();
     int unbuffered() { return _flags & _IO_UNBUFFERED ? 1 : 0; }
     int linebuffered() { return _flags & _IO_LINE_BUF ? 1 : 0; }
     void unbuffered(int i)
@@ -366,19 +367,14 @@ struct streambuf : public _IO_FILE { // protected??
     // Allocate a buffer if needed; use _shortbuf if appropriate.
     void allocbuf() { if (base() == NULL) doallocbuf(); }
     void doallocbuf();
-    virtual int sync();
-    virtual int pbackfail(int c);
-    virtual streambuf* setbuf(char* p, int len);
     int in_avail() { return _IO_read_end - _IO_read_ptr; }
     int out_waiting() { return _IO_write_ptr - _IO_write_base; }
-    virtual streamsize xsputn(const char* s, streamsize n);
     streamsize sputn(const char* s, streamsize n) { return xsputn(s, n); }
     streamsize padn(char pad, streamsize n) { return _IO_padn(this, pad, n); }
-    virtual streamsize xsgetn(char* s, streamsize n);
     streamsize sgetn(char* s, streamsize n) { return _IO_sgetn(this, s, n); }
     int ignore(int);
-    virtual int get_column();
-    virtual int set_column(int);
+    int get_column();
+    int set_column(int);
     long sgetline(char* buf, _IO_size_t n, char delim, int putback_delim);
     int sputc(int c) { return _IO_putc(c, this); }
     int sbumpc() { return _IO_getc(this); }
@@ -397,10 +393,10 @@ struct streambuf : public _IO_FILE { // protected??
     void column(int c);  // Set column number of put pointer to c.
 #endif
     virtual streamsize sys_read(char* buf, streamsize size);
-    virtual streampos sys_seek(streamoff, _seek_dir);
     virtual streamsize sys_write(const char*, streamsize);
-    virtual int sys_stat(void*); // Actually, a (struct stat*)
+    virtual streampos sys_seek(streamoff, _seek_dir);
     virtual int sys_close();
+    virtual int sys_stat(void*); // Actually, a (struct stat*)
 };
 
 // A backupbuf is a streambuf with full backup and savepoints on reading.
@@ -414,7 +410,9 @@ class filebuf : public streambuf {
     filebuf();
     filebuf(int fd);
     filebuf(int fd, char* p, int len);
+#if !_IO_UNIFIED_JUMPTABLES
     static filebuf *__new();
+#endif
     ~filebuf();
     filebuf* attach(int fd);
     filebuf* open(const char *filename, const char *mode);
@@ -436,20 +434,34 @@ class filebuf : public streambuf {
     char* cur_ptr() { return is_reading() ?  gptr() : pptr(); }
     /* System's idea of pointer */
     char* file_ptr() { return eGptr(); }
-    int do_write(const char *data, int to_do);
     // Low-level operations (Usually invoke system calls.)
     virtual streamsize sys_read(char* buf, streamsize size);
     virtual streampos sys_seek(streamoff, _seek_dir);
     virtual streamsize sys_write(const char*, streamsize);
     virtual int sys_stat(void*); // Actually, a (struct stat*)
     virtual int sys_close();
+#if 0
+    virtual uflow;
+    virtual showmany;
+#endif
 };
 
 inline void ios::init(streambuf* sb, ostream* tie_to) {
 		_state = sb ? ios::goodbit : ios::badbit; _exceptions=0;
 		_strbuf=sb; _tie = tie_to; _width=0; _fill=' ';
-		_flags=ios::skipws|ios::dec; _precision=6; _arrays = 0; }
+#ifdef _IO_NEW_STREAMS
+		_flags=ios::skipws|ios::dec;
+#else
+		_flags=ios::skipws|ios::dec|ios::dont_close;
+#endif
+		_precision=6; _arrays = 0; }
+
+inline ios::ios(streambuf* sb, ostream* tie_to) { init(sb, tie_to); }
 
 inline ios::~ios() {
-    if (!(_flags & (unsigned int)ios::dont_close)) delete rdbuf(); }
+#ifndef _IO_NEW_STREAMS
+    if (!(_flags & (unsigned int)ios::dont_close)) delete rdbuf();
+#endif
+    if (_arrays) delete [] _arrays;
+}
 #endif /* _STREAMBUF_H */
