@@ -1,6 +1,7 @@
 # We don't need -fpic on the alpha, so let's install both the shared and
 # non-shared versions.
 
-LIBS     = $(ARLIB) $(SHLIB)
-DEPLIBS  = ../libstdc++.so
+LIBS     = $(ARLIB) $(SHLIB) $(SHLINK)
+DEPLIBS  = ../$(SHLIB)
+LDLIBS   = -L.. -lstdc++ -rpath ..
 SHDEPS   = -lm
